/* 
 * Created on 2-jul-2006 by ronald.blankendaal
 * 
 * @file $RCSfile: EditTemplateDialog.java,v $
 * @version $Revision: 1.17 $ 
 * @author $Author: ronald $ (last checked in by) 
 * @date $Date: 2006/09/12 18:42:06 $ (UTC date of last check in)
 */

package com.gui;

import java.io.File;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import com.db.Database;
import com.model.Autoexec;
import com.model.Configuration;
import com.model.DosboxVersion;
import com.model.Mount;
import com.model.Template;
import com.util.ListUtils;
import swing2swt.layout.BorderLayout;


public class EditTemplateDialog extends Dialog {

	private Button glide;
  private Text glideport;
  private Text vsyncrate;
  private Combo vsyncmode;
  private Combo videoram;
  private Combo svgachipset;
  private Text overscan;
  private Combo mt32rate;
  private Combo pixelshader;
  private TabItem infoTabItem;
  private Text title;
  private Button reloadSettingsButton;
  private Combo dbversion;
  private Button waitonerror;
  private Button exit;
  private Combo priority_inactive;
  private Combo priority_active;
  private Text captures;
  private Text language;
  private Combo windowresolution;
  private Combo fullresolution;
  private Combo frameskip;
  private Combo output;
  private Button fullscreen;
  private Button fulldouble;
  private Combo scaler;
  private Button aspect;
  private Combo machine;
  private Combo cycles_down;
  private Combo cycles_up;
  private Combo cycles;
  private Combo core;
  private Button timesynched;
  private Button xms;
  private Button ems;
  private Combo umb;
  private Combo memsize;
  private Button nosound;
  private Button mixer;
  private Button pcspeaker;
  private Button disney;
  private Button gus;
  private Combo tandy;
  private Combo tandyrate;
  private Text midi_config;
  private Combo device;
  private Combo mpu401;
  private Text ultradir;
  private Combo gusrate;
  private Combo irq2;
  private Combo dma2;
  private Combo dma1;
  private Combo irq1;
  private Combo gusbase;
  private Combo pcrate;
  private Combo hdma;
  private Combo dma;
  private Combo irq;
  private Combo sbbase;
  private Combo oplrate;
  private Combo oplmode;
  private Combo sbtype;
  private Combo prebuffer;
  private Combo blocksize;
  private Combo rate;
  private Button usescancodes;
  private Button autolock;
  private Text serial4;
  private Text serial3;
  private Text serial2;
  private Text serial1;
  private Combo joysticktype;
  private Text mapperfile;
  private Combo sensitivity;
  private Button ipx;
  private List mountingpoints;
  private Button booterButton;
  private Button dosButton;
  
  protected Shell shell;
  private java.util.List<DosboxVersion> dbversionsList;
  private Database db;
  private Configuration templateConf; 
  protected Object result;
  
  /**
   * Create the dialog
   * @param parent
   * @param style
   */
  public EditTemplateDialog(Shell parent, int style) {
    super(parent, style);
  }

  /**
   * Create the dialog
   * @param parent
   */
  public EditTemplateDialog(Shell parent) {
    this(parent, SWT.NONE);
  }
  
  public void setDatabase(Database db) {
    this.db = db;
  }
  
  public void setTemplate(Template template) {
    this.result = template;
  }

  /**
   * Open the dialog
   * @return the result
   */
  public Object open() {
    createContents();
    shell.open();
    shell.layout();
    Display display = getParent().getDisplay();
    while (!shell.isDisposed()) {
      if (!display.readAndDispatch())
        display.sleep();
    }
    return result;
  }

  /**
   * Create contents of the dialog
   */
  protected void createContents() {
    dbversionsList = db.readDosboxVersionsList();
    
    shell = new Shell(getParent(), SWT.TITLE | SWT.CLOSE | SWT.BORDER | SWT.RESIZE | SWT.APPLICATION_MODAL);
    shell.setLayout(new BorderLayout(0, 0));
    shell.setSize(610, 475);
    
    if (result != null) {
      // meaning we are essentially editing an existing template
      shell.setText("Edit Template " + ((Template)result).getTitle() + " (" + ((Template)result).getId() + ")");
    } else {
      shell.setText("Add Template");
    }

    final TabFolder tabFolder = new TabFolder(shell, SWT.NONE);

    infoTabItem = new TabItem(tabFolder, SWT.NONE);
    infoTabItem.setText("Info");

    final Composite composite = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    composite.setLayout(gridLayout);
    infoTabItem.setControl(composite);

    final Label titleLabel = new Label(composite, SWT.NONE);
    titleLabel.setText("Title");

    title = new Text(composite, SWT.BORDER);
    title.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    
    final TabItem generalTabItem = new TabItem(tabFolder, SWT.NONE);
    generalTabItem.setText("General");

    final Composite composite_1 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_1 = new GridLayout();
    gridLayout_1.numColumns = 3;
    composite_1.setLayout(gridLayout_1);
    generalTabItem.setControl(composite_1);

    final Label dosboxVersionLabel = new Label(composite_1, SWT.NONE);
    dosboxVersionLabel.setText("Dosbox version");

    dbversion = new Combo(composite_1, SWT.READ_ONLY);
    dbversion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    
    reloadSettingsButton = new Button(composite_1, SWT.NONE);
    reloadSettingsButton.setText("Reload Settings");
    reloadSettingsButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        DosboxVersion selectedDBVersion = dbversionsList.get(dbversion.getSelectionIndex());
        templateConf = new Configuration(selectedDBVersion.getConfigFile());
        selectSettingsByConfiguration(templateConf);
        // enabled 'exit afterwards' hardcoded, by default
        exit.setSelection(true);
      }
    });
    
    final Label label = new Label(composite_1, SWT.SEPARATOR | SWT.HORIZONTAL);
    label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 3, 1));
    new Label(composite_1, SWT.NONE);

    final Label activeLabel = new Label(composite_1, SWT.NONE);
    activeLabel.setText("Active");

    final Label inactiveLabel = new Label(composite_1, SWT.NONE);
    inactiveLabel.setText("Inactive");

    final Label priorityLabel = new Label(composite_1, SWT.NONE);
    priorityLabel.setText("Priority");

    priority_active = new Combo(composite_1, SWT.READ_ONLY);
    priority_active.setItems(new String[] {"lowest", "lower", "normal", "higher", "highest", "pause"});
    final GridData gridData = new GridData(SWT.LEFT, SWT.CENTER, true, false);
    gridData.minimumWidth = 150;
    priority_active.setLayoutData(gridData);

    priority_inactive = new Combo(composite_1, SWT.READ_ONLY);
    priority_inactive.setItems(new String[] {"lowest", "lower", "normal", "higher", "highest", "pause"});
    priority_inactive.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));

    final Label waitOnErrorLabel = new Label(composite_1, SWT.NONE);
    waitOnErrorLabel.setText("Wait On Error");

    waitonerror = new Button(composite_1, SWT.CHECK);
    new Label(composite_1, SWT.NONE);

    final Label exitAfterwardsLabel = new Label(composite_1, SWT.NONE);
    exitAfterwardsLabel.setText("Exit afterwards");

    exit = new Button(composite_1, SWT.CHECK);
    new Label(composite_1, SWT.NONE);

    final Label languageFileLabel = new Label(composite_1, SWT.NONE);
    languageFileLabel.setLayoutData(new GridData());
    languageFileLabel.setText("Language File");

    language = new Text(composite_1, SWT.BORDER);
    language.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
    
    final TabItem displayTabItem = new TabItem(tabFolder, SWT.NONE);
    displayTabItem.setText("Display");

    final Composite composite_2 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_5 = new GridLayout();
    gridLayout_5.numColumns = 4;
    composite_2.setLayout(gridLayout_5);
    displayTabItem.setControl(composite_2);

    final Label outputLabel = new Label(composite_2, SWT.NONE);
    outputLabel.setText("Output");

    output = new Combo(composite_2, SWT.READ_ONLY);
    output.setItems(new String[] {"ddraw", "direct3d", "overlay", "opengl", "openglhq", "openglnb", "surface"});
    output.setVisibleItemCount(10);
    output.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label frameskipLabel = new Label(composite_2, SWT.NONE);
    frameskipLabel.setText("Frameskip");

    frameskip = new Combo(composite_2, SWT.READ_ONLY);
    frameskip.setItems(new String[] {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"});
    frameskip.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label scalerLabel = new Label(composite_2, SWT.NONE);
    scalerLabel.setText("Scaler");

    scaler = new Combo(composite_2, SWT.READ_ONLY);
    scaler.setItems(new String[] {"none", "normal2x", "normal3x", "advmame2x", "advmame3x", "advinterp2x", "advinterp3x", "tv2x", "tv3x",
    		"rgb2x", "rgb3x", "scan2x", "scan3x", "supereagle", "super2xsai", "2xsai", "hq2x", "hq3x", "hardware2x", "hardware3x"});
    scaler.setVisibleItemCount(15);
    scaler.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    
    final Label pixelshaderLabel = new Label(composite_2, SWT.NONE);
    pixelshaderLabel.setLayoutData(new GridData());
    pixelshaderLabel.setText("Pixelshader");

    pixelshader = new Combo(composite_2, SWT.READ_ONLY);
    pixelshader.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    pixelshader.setVisibleItemCount(15);
    pixelshader.setItems(new String[] {"none", 
    		"2xsai.fx", "2xsai_srgb.fx", "bilinear.fx", "cartoon.fx", "gs2x.fx", 
    		"gs4x.fx", "gs4xcolor.fx", "gs4xcolorfilter.fx", "gs4xcolorscale.fx", "gs4xfilter.fx", "gs4xscale.fx", 
    		"matrix.fx", "mcgreen.fx", "mcorange.fx", "point.fx", "scale2x.fx", "scale2x_ps14.fx", "sketch.fx"});

    final Label fullscreenLabel = new Label(composite_2, SWT.NONE);
    fullscreenLabel.setText("Fullscreen");

    fullscreen = new Button(composite_2, SWT.CHECK);
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label doubleBufferingLabel = new Label(composite_2, SWT.NONE);
    doubleBufferingLabel.setText("Double Buffering");

    fulldouble = new Button(composite_2, SWT.CHECK);
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label resolutionLabel = new Label(composite_2, SWT.NONE);
    resolutionLabel.setText("Fullscreen Resolution");

    fullresolution = new Combo(composite_2, SWT.READ_ONLY);
    fullresolution.setItems(new String[] {"original", "320x200", "640x480", "800x600", "1024x768", "1280x768", "1280x960", "1280x1024"});
    fullresolution.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label windowResolutionLabel = new Label(composite_2, SWT.NONE);
    windowResolutionLabel.setText("Window Resolution");

    windowresolution = new Combo(composite_2, SWT.READ_ONLY);
    windowresolution.setItems(new String[] {"original", "320x200", "640x480", "800x600", "1024x768", "1280x768", "1280x960", "1280x1024"});
    windowresolution.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label aspectCorrectionLabel = new Label(composite_2, SWT.NONE);
    aspectCorrectionLabel.setText("Aspect Correction");

    aspect = new Button(composite_2, SWT.CHECK);
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final TabItem machineTabItem = new TabItem(tabFolder, SWT.NONE);
    machineTabItem.setText("Machine");

    final Composite composite_3 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_2 = new GridLayout();
    composite_3.setLayout(gridLayout_2);
    machineTabItem.setControl(composite_3);

    final Group cpuGroup = new Group(composite_3, SWT.NONE);
    cpuGroup.setText("CPU");
    final GridLayout gridLayout_3 = new GridLayout();
    gridLayout_3.numColumns = 6;
    cpuGroup.setLayout(gridLayout_3);

    final Label overscanLabel = new Label(composite_2, SWT.NONE);
    overscanLabel.setText("Overscan");

    overscan = new Text(composite_2, SWT.BORDER);
    overscan.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label svgaChipsetLabel = new Label(composite_2, SWT.NONE);
    svgaChipsetLabel.setText("SVGA Chipset");

    svgachipset = new Combo(composite_2, SWT.READ_ONLY);
    svgachipset.setItems(new String[] {"none", "et3000", "et4000", "et4000new", "pvga1a", "s3"});
    svgachipset.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label videoRamLabel = new Label(composite_2, SWT.NONE);
    videoRamLabel.setText("Video RAM in KB");

    videoram = new Combo(composite_2, SWT.READ_ONLY);
    videoram.setItems(new String[] {"256", "512", "1024", "2048", "3072", "4096"});
    videoram.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label vsyncModeLabel = new Label(composite_2, SWT.NONE);
    vsyncModeLabel.setText("Vsync Mode");

    vsyncmode = new Combo(composite_2, SWT.READ_ONLY);
    vsyncmode.setItems(new String[] {"off", "on", "force"});
    vsyncmode.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label vsyncRateLabel = new Label(composite_2, SWT.NONE);
    vsyncRateLabel.setLayoutData(new GridData());
    vsyncRateLabel.setText("Vsync Rate");

    vsyncrate = new Text(composite_2, SWT.BORDER);
    vsyncrate.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label glideLabel = new Label(composite_2, SWT.NONE);
    glideLabel.setText("Glide");

    glide = new Button(composite_2, SWT.CHECK);
    glide.addSelectionListener(new SelectionAdapter() {
    	public void widgetSelected(final SelectionEvent e) {
    		glideport.setEnabled(glide.getSelection());
    	}
    });

    final Label glidePortLabel = new Label(composite_2, SWT.NONE);
    glidePortLabel.setText("Glide port");

    glideport = new Text(composite_2, SWT.BORDER);
    glideport.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label machineLabel = new Label(cpuGroup, SWT.NONE);
    machineLabel.setText("Machine");

    machine = new Combo(cpuGroup, SWT.READ_ONLY);
    machine.setItems(new String[] {"cga", "hercules", "pcjr", "tandy", "vga"});
    machine.setLayoutData(new GridData());
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);

    final Label coreLabel = new Label(cpuGroup, SWT.NONE);
    coreLabel.setText("Core");

    core = new Combo(cpuGroup, SWT.READ_ONLY);
    core.setItems(new String[] {"dynamic", "full", "normal", "simple", "auto"});
    core.setLayoutData(new GridData());
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);

    final Label cyclesLabel = new Label(cpuGroup, SWT.NONE);
    cyclesLabel.setText("Cycles");

    cycles = new Combo(cpuGroup, SWT.NONE);
    cycles.setItems(new String[] {"500", "750", "1000", "2000", "3000", "4000", "5000", "7500", "10000", "12500", "15000", "17500", "20000", "25000", "30000", "32500", "35000", "40000", "45000", "50000", "55000", "60000", "auto", "max"});
    cycles.setLayoutData(new GridData());
    cycles.setVisibleItemCount(15);

    final Label upLabel = new Label(cpuGroup, SWT.NONE);
    upLabel.setLayoutData(new GridData());
    upLabel.setText("Up");

    cycles_up = new Combo(cpuGroup, SWT.NONE);
    cycles_up.setItems(new String[] {"20", "50", "100", "500", "1000", "2000", "5000", "10000"});
    cycles_up.setLayoutData(new GridData());

    final Label downLabel = new Label(cpuGroup, SWT.NONE);
    downLabel.setLayoutData(new GridData());
    downLabel.setText("Down");

    cycles_down = new Combo(cpuGroup, SWT.NONE);
    cycles_down.setItems(new String[] {"20", "50", "100", "500", "1000", "2000", "5000", "10000"});
    cycles_down.setLayoutData(new GridData());

    final Label timesynchedLabel = new Label(cpuGroup, SWT.NONE);
    timesynchedLabel.setText("Timesynched");

    timesynched = new Button(cpuGroup, SWT.CHECK);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);

    final Group memoryGroup = new Group(composite_3, SWT.NONE);
    memoryGroup.setText("Memory");
    memoryGroup.setLayoutData(new GridData());
    final GridLayout gridLayout_4 = new GridLayout();
    gridLayout_4.numColumns = 2;
    memoryGroup.setLayout(gridLayout_4);

    final Label sizeLabel = new Label(memoryGroup, SWT.NONE);
    sizeLabel.setText("Size");

    memsize = new Combo(memoryGroup, SWT.READ_ONLY);
    memsize.setItems(new String[] {"0", "1", "2", "4", "8", "16", "32", "63"});
    memsize.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label xmsLabel = new Label(memoryGroup, SWT.NONE);
    xmsLabel.setText("XMS");

    xms = new Button(memoryGroup, SWT.CHECK);

    final Label emsLabel = new Label(memoryGroup, SWT.NONE);
    emsLabel.setText("EMS");

    ems = new Button(memoryGroup, SWT.CHECK);

    final Label umbLabel = new Label(memoryGroup, SWT.NONE);
    umbLabel.setText("UMB");

    umb = new Combo(memoryGroup, SWT.READ_ONLY);
    umb.setItems(new String[] {"false", "true", "max"});
    umb.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final TabItem audioTabItem = new TabItem(tabFolder, SWT.NONE);
    audioTabItem.setText("Audio");

    final Composite composite_4 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_9 = new GridLayout();
    gridLayout_9.numColumns = 3;
    composite_4.setLayout(gridLayout_9);
    audioTabItem.setControl(composite_4);

    final Group generalGroup = new Group(composite_4, SWT.NONE);
    generalGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    final GridLayout gridLayout_6 = new GridLayout();
    gridLayout_6.numColumns = 2;
    generalGroup.setLayout(gridLayout_6);
    generalGroup.setText("General");

    final Label silentModeLabel = new Label(generalGroup, SWT.NONE);
    silentModeLabel.setText("Silent Mode");

    nosound = new Button(generalGroup, SWT.CHECK);

    final Label sampleRateLabel = new Label(generalGroup, SWT.NONE);
    sampleRateLabel.setText("Sample Rate");

    rate = new Combo(generalGroup, SWT.READ_ONLY);
    rate.setItems(new String[] {"11025", "22050", "44100"});
    rate.setLayoutData(new GridData());

    final Label blockSizeLabel = new Label(generalGroup, SWT.NONE);
    blockSizeLabel.setText("Block Size");

    blocksize = new Combo(generalGroup, SWT.READ_ONLY);
    blocksize.setItems(new String[] {"512", "1024", "2048", "3072", "4096", "8192"});
    blocksize.setLayoutData(new GridData());

    final Label prebufferLabel = new Label(generalGroup, SWT.NONE);
    prebufferLabel.setText("Prebuffer");

    prebuffer = new Combo(generalGroup, SWT.NONE);
    prebuffer.setItems(new String[] {"10"});
    prebuffer.setLayoutData(new GridData());

    final Label mpu401Label = new Label(generalGroup, SWT.NONE);
    mpu401Label.setText("MPU-401");

    mpu401 = new Combo(generalGroup, SWT.READ_ONLY);
    mpu401.setItems(new String[] {"none", "intelligent", "uart"});
    mpu401.setLayoutData(new GridData());

    final Label midiDeviceLabel = new Label(generalGroup, SWT.NONE);
    midiDeviceLabel.setText("Midi Device");

    device = new Combo(generalGroup, SWT.READ_ONLY);
    device.setItems(new String[] {"alsa", "default", "coreaudio", "mt32", "none", "oss", "win32"});
    device.setLayoutData(new GridData());

    final Label configLabel = new Label(generalGroup, SWT.NONE);
    configLabel.setText("Config");

    midi_config = new Text(generalGroup, SWT.BORDER);
    midi_config.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label mt32RateLabel = new Label(generalGroup, SWT.NONE);
    mt32RateLabel.setText("MT-32 Rate");

    mt32rate = new Combo(generalGroup, SWT.READ_ONLY);
    mt32rate.setItems(new String[] {"11025", "22050", "44100"});
    mt32rate.setLayoutData(new GridData());
    
    final Group soundblasterGroup = new Group(composite_4, SWT.NONE);
    soundblasterGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    soundblasterGroup.setText("Soundblaster");
    final GridLayout gridLayout_7 = new GridLayout();
    gridLayout_7.numColumns = 2;
    soundblasterGroup.setLayout(gridLayout_7);

    final Label typeLabel = new Label(soundblasterGroup, SWT.NONE);
    typeLabel.setText("Type");

    sbtype = new Combo(soundblasterGroup, SWT.READ_ONLY);
    sbtype.setItems(new String[] {"none", "sb1", "sb2", "sbpro1", "sbpro2", "sb16"});
    sbtype.setLayoutData(new GridData());

    final Label oplRateLabel = new Label(soundblasterGroup, SWT.NONE);
    oplRateLabel.setText("OPL Rate");

    oplrate = new Combo(soundblasterGroup, SWT.READ_ONLY);
    oplrate.setItems(new String[] {"11025", "22050", "44100"});
    oplrate.setLayoutData(new GridData());

    final Label oplModeLabel = new Label(soundblasterGroup, SWT.NONE);
    oplModeLabel.setLayoutData(new GridData());
    oplModeLabel.setText("OPL Mode");

    oplmode = new Combo(soundblasterGroup, SWT.READ_ONLY);
    oplmode.setItems(new String[] {"auto", "cms", "opl2", "dualopl2", "opl3"});
    oplmode.setLayoutData(new GridData());

    final Label addressLabel = new Label(soundblasterGroup, SWT.NONE);
    addressLabel.setText("Address");

    sbbase = new Combo(soundblasterGroup, SWT.READ_ONLY);
    sbbase.setItems(new String[] {"210", "220", "240", "260", "280"});
    sbbase.setLayoutData(new GridData());

    final Label irqLabel = new Label(soundblasterGroup, SWT.NONE);
    irqLabel.setText("IRQ");

    irq = new Combo(soundblasterGroup, SWT.READ_ONLY);
    irq.setItems(new String[] {"3", "5", "7", "10", "11"});
    irq.setLayoutData(new GridData());

    final Label dmaLabel = new Label(soundblasterGroup, SWT.NONE);
    dmaLabel.setText("DMA");

    dma = new Combo(soundblasterGroup, SWT.READ_ONLY);
    dma.setItems(new String[] {"0", "1", "3"});
    dma.setLayoutData(new GridData());

    final Label hdmaLabel = new Label(soundblasterGroup, SWT.NONE);
    hdmaLabel.setLayoutData(new GridData());
    hdmaLabel.setText("HDMA");

    hdma = new Combo(soundblasterGroup, SWT.READ_ONLY);
    hdma.setItems(new String[] {"5", "6", "7"});
    hdma.setLayoutData(new GridData());

    final Label mixerLabel = new Label(soundblasterGroup, SWT.NONE);
    mixerLabel.setText("Mixer");

    mixer = new Button(soundblasterGroup, SWT.CHECK);

    final Group gusGroup = new Group(composite_4, SWT.NONE);
    gusGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    gusGroup.setText("Gravis Ultrasound");
    final GridLayout gridLayout_8 = new GridLayout();
    gridLayout_8.numColumns = 2;
    gusGroup.setLayout(gridLayout_8);

    final Label enableLabel_1 = new Label(gusGroup, SWT.NONE);
    enableLabel_1.setLayoutData(new GridData());
    enableLabel_1.setText("Enable");

    gus = new Button(gusGroup, SWT.CHECK);
    gus.setLayoutData(new GridData());

    final Label sampleRateLabel_2 = new Label(gusGroup, SWT.NONE);
    sampleRateLabel_2.setText("Sample Rate");

    gusrate = new Combo(gusGroup, SWT.READ_ONLY);
    gusrate.setItems(new String[] {"11025", "22050", "44100"});
    gusrate.setLayoutData(new GridData());

    final Label addressLabel_1 = new Label(gusGroup, SWT.NONE);
    addressLabel_1.setText("Address");

    gusbase = new Combo(gusGroup, SWT.READ_ONLY);
    gusbase.setItems(new String[] {"210", "220", "240", "260", "280"});
    gusbase.setLayoutData(new GridData());

    final Label irqLabel_1 = new Label(gusGroup, SWT.NONE);
    irqLabel_1.setText("IRQ1");

    irq1 = new Combo(gusGroup, SWT.READ_ONLY);
    irq1.setItems(new String[] {"3", "5", "7", "10", "11"});
    irq1.setLayoutData(new GridData());

    final Label irq2Label = new Label(gusGroup, SWT.NONE);
    irq2Label.setText("IRQ2");

    irq2 = new Combo(gusGroup, SWT.READ_ONLY);
    irq2.setItems(new String[] {"3", "5", "7", "10", "11"});
    irq2.setLayoutData(new GridData());

    final Label dmaLabel_1 = new Label(gusGroup, SWT.NONE);
    dmaLabel_1.setText("DMA1");

    dma1 = new Combo(gusGroup, SWT.READ_ONLY);
    dma1.setItems(new String[] {"0", "1", "3"});
    dma1.setLayoutData(new GridData());

    final Label hdmaLabel_1 = new Label(gusGroup, SWT.NONE);
    hdmaLabel_1.setLayoutData(new GridData());
    hdmaLabel_1.setText("DMA2");

    dma2 = new Combo(gusGroup, SWT.READ_ONLY);
    dma2.setItems(new String[] {"0", "1", "3"});
    dma2.setLayoutData(new GridData());

    final Label ultradirLabel = new Label(gusGroup, SWT.NONE);
    ultradirLabel.setText("Ultradir");

    ultradir = new Text(gusGroup, SWT.BORDER);
    ultradir.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group speakerGroup = new Group(composite_4, SWT.NONE);
    speakerGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    speakerGroup.setText("PC Speaker");
    final GridLayout gridLayout_10 = new GridLayout();
    gridLayout_10.numColumns = 2;
    speakerGroup.setLayout(gridLayout_10);

    final Label enableLabel = new Label(speakerGroup, SWT.NONE);
    enableLabel.setText("Enable");

    pcspeaker = new Button(speakerGroup, SWT.CHECK);

    final Label sampleRateLabel_1 = new Label(speakerGroup, SWT.NONE);
    sampleRateLabel_1.setText("Sample Rate");

    pcrate = new Combo(speakerGroup, SWT.READ_ONLY);
    pcrate.setItems(new String[] {"11025", "22050", "44100"});
    pcrate.setLayoutData(new GridData());

    final Group tandyGroup = new Group(composite_4, SWT.NONE);
    tandyGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    tandyGroup.setText("Tandy");
    final GridLayout gridLayout_11 = new GridLayout();
    gridLayout_11.numColumns = 2;
    tandyGroup.setLayout(gridLayout_11);

    final Label enableLabel_2 = new Label(tandyGroup, SWT.NONE);
    enableLabel_2.setText("Enable");

    tandy = new Combo(tandyGroup, SWT.READ_ONLY);
    tandy.setItems(new String[] {"auto", "off", "on"});
    tandy.setLayoutData(new GridData());

    final Label sampleRateLabel_1_1 = new Label(tandyGroup, SWT.NONE);
    sampleRateLabel_1_1.setText("Sample Rate");

    tandyrate = new Combo(tandyGroup, SWT.READ_ONLY);
    tandyrate.setItems(new String[] {"11025", "22050", "44100"});
    tandyrate.setLayoutData(new GridData());

    final Group disneyGroup = new Group(composite_4, SWT.NONE);
    disneyGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    disneyGroup.setText("Disney Sound Source");
    final GridLayout gridLayout_12 = new GridLayout();
    gridLayout_12.numColumns = 2;
    disneyGroup.setLayout(gridLayout_12);

    final Label enableLabel_3 = new Label(disneyGroup, SWT.NONE);
    enableLabel_3.setText("Enable");

    disney = new Button(disneyGroup, SWT.CHECK);

    final TabItem inputTabItem = new TabItem(tabFolder, SWT.NONE);
    inputTabItem.setText("I/O");

    final Composite composite_5 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_17 = new GridLayout();
    gridLayout_17.numColumns = 3;
    composite_5.setLayout(gridLayout_17);
    inputTabItem.setControl(composite_5);

    final Group mouseGroup = new Group(composite_5, SWT.NONE);
    mouseGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    mouseGroup.setText("Mouse");
    final GridLayout gridLayout_13 = new GridLayout();
    gridLayout_13.numColumns = 2;
    mouseGroup.setLayout(gridLayout_13);

    final Label autolockLabel = new Label(mouseGroup, SWT.NONE);
    autolockLabel.setText("Autolock");

    autolock = new Button(mouseGroup, SWT.CHECK);

    final Label sensitivityLabel = new Label(mouseGroup, SWT.NONE);
    sensitivityLabel.setText("Sensitivity");

    sensitivity = new Combo(mouseGroup, SWT.READ_ONLY);
    sensitivity.setItems(new String[] {"10", "20", "30", "40", "50", "60", "70", "80", "90", "100", "125", "150", "175", "200", "250", "300", "350", "400", "450", "500", "550", "600", "700", "800", "900", "1000"});
    sensitivity.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));

    final Group keyboardGroup = new Group(composite_5, SWT.NONE);
    keyboardGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    keyboardGroup.setText("Keyboard");
    final GridLayout gridLayout_14 = new GridLayout();
    gridLayout_14.numColumns = 2;
    keyboardGroup.setLayout(gridLayout_14);

    final Label useScancodesLabel = new Label(keyboardGroup, SWT.NONE);
    useScancodesLabel.setText("Use Scancodes");

    usescancodes = new Button(keyboardGroup, SWT.CHECK);

    final Label mapperFileLabel = new Label(keyboardGroup, SWT.NONE);
    mapperFileLabel.setText("Mapper File");

    mapperfile = new Text(keyboardGroup, SWT.BORDER);
    mapperfile.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group joystickGroup = new Group(composite_5, SWT.NONE);
    joystickGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    joystickGroup.setText("Joystick");
    final GridLayout gridLayout_15 = new GridLayout();
    gridLayout_15.numColumns = 2;
    joystickGroup.setLayout(gridLayout_15);

    final Label typeLabel_1 = new Label(joystickGroup, SWT.NONE);
    typeLabel_1.setText("Type");

    joysticktype = new Combo(joystickGroup, SWT.READ_ONLY);
    joysticktype.setItems(new String[] {"none", "2axis", "4axis", "ch", "fcs"});
    final GridData gridData_1 = new GridData(SWT.FILL, SWT.CENTER, true, false);
    joysticktype.setLayoutData(gridData_1);

    final Group modemGroup = new Group(composite_5, SWT.NONE);
    modemGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
    modemGroup.setText("Modem");
    final GridLayout gridLayout_16 = new GridLayout();
    gridLayout_16.numColumns = 2;
    modemGroup.setLayout(gridLayout_16);

    final Label serial1Label = new Label(modemGroup, SWT.NONE);
    serial1Label.setText("Serial 1");

    serial1 = new Text(modemGroup, SWT.BORDER);
    serial1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label serial2Label = new Label(modemGroup, SWT.NONE);
    serial2Label.setText("Serial 2");

    serial2 = new Text(modemGroup, SWT.BORDER);
    serial2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label serial3Label = new Label(modemGroup, SWT.NONE);
    serial3Label.setText("Serial 3");

    serial3 = new Text(modemGroup, SWT.BORDER);
    serial3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label serial4Label = new Label(modemGroup, SWT.NONE);
    serial4Label.setText("Serial 4");

    serial4 = new Text(modemGroup, SWT.BORDER);
    serial4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    
    final Group networkGroup = new Group(composite_5, SWT.NONE);
    networkGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    networkGroup.setText("Network");
    final GridLayout gridLayout_20 = new GridLayout();
    gridLayout_20.numColumns = 2;
    networkGroup.setLayout(gridLayout_20);

    final Label ipxLabel = new Label(networkGroup, SWT.NONE);
    ipxLabel.setText("Enable IPX");

    ipx = new Button(networkGroup, SWT.CHECK);
    new Label(composite_5, SWT.NONE);

    final TabItem mountingTabItem = new TabItem(tabFolder, SWT.NONE);
    mountingTabItem.setText("Mounting");

    final Composite composite_6 = new Composite(tabFolder, SWT.NONE);
    composite_6.setLayout(new GridLayout());
    mountingTabItem.setControl(composite_6);

    final Group mountGroup = new Group(composite_6, SWT.NONE);
    mountGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    mountGroup.setText("Mounting Overview");
    final GridLayout gridLayout_19 = new GridLayout();
    gridLayout_19.numColumns = 2;
    mountGroup.setLayout(gridLayout_19);

    mountingpoints = new List(mountGroup, SWT.V_SCROLL | SWT.BORDER);
    final GridData gridData_2 = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 3);
    mountingpoints.setLayoutData(gridData_2);

    final Button addButton = new Button(mountGroup, SWT.NONE);
    addButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
    addButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        final EditMountDialog addMountDialog = new EditMountDialog(shell);
        java.util.List<Character> freeDriveletters =
        	Mount.getFreeDriveletters(booterButton.getSelection(), mountingpoints.getItems());
        if (freeDriveletters.size() > 0)
        	addMountDialog.setDefaultDriveletter(freeDriveletters.get(0));
        String mount = (String)addMountDialog.open();
        if (mount != null) {
          mountingpoints.add(mount);
          mountingpoints.select(mountingpoints.getItemCount() - 1);
        }
      }
    });
    addButton.setText("Add");

    final Button editButton = new Button(mountGroup, SWT.NONE);
    editButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
    editButton.setText("Edit");
    editButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        if (mountingpoints.getSelectionIndex() != -1) {
          final EditMountDialog editMountDialog = new EditMountDialog(shell);
          editMountDialog.setMount(mountingpoints.getItem(mountingpoints.getSelectionIndex()));
          String mount = (String)editMountDialog.open();
          if (mount != null) {
            mountingpoints.setItem(mountingpoints.getSelectionIndex(), mount);
          }
        }
      }
    });

    final Button removeButton = new Button(mountGroup, SWT.NONE);
    removeButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
    removeButton.setText("Remove");
    removeButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        if (mountingpoints.getSelectionIndex() != -1) {
          mountingpoints.remove(mountingpoints.getSelectionIndex());
        } else if (mountingpoints.getItemCount() == 1) {
        	mountingpoints.removeAll();
        }
      }
    });

    final Group executeGroup = new Group(composite_6, SWT.NONE);
    executeGroup.setLayoutData(new GridData());
    executeGroup.setText("Execute");
    final GridLayout gridLayout_18 = new GridLayout();
    executeGroup.setLayout(gridLayout_18);

    dosButton = new Button(executeGroup, SWT.RADIO);
    dosButton.setText("Dos");

    final Label label_1 = new Label(executeGroup, SWT.SEPARATOR | SWT.HORIZONTAL);
    label_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));

    booterButton = new Button(executeGroup, SWT.RADIO);
    booterButton.setText("Booter");

    final Composite composite_7 = new Composite(shell, SWT.NONE);
    composite_7.setLayout(new RowLayout());
    composite_7.setLayoutData(BorderLayout.SOUTH);

    final Label capturesLabel = new Label(composite_1, SWT.NONE);
    capturesLabel.setLayoutData(new GridData());
    capturesLabel.setText("Captures");

    captures = new Text(composite_1, SWT.BORDER);
    captures.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
    captures.setEditable(false);

    final Label defaultLabel = new Label(composite, SWT.NONE);
    defaultLabel.setText("Default");

    final Button defaultButton = new Button(composite, SWT.CHECK);

    final Button okButton = new Button(composite_7, SWT.NONE);
    shell.setDefaultButton(okButton);
    final RowData rowData = new RowData();
    rowData.width = 80;
    okButton.setLayoutData(rowData);
    okButton.setText("OK");
    okButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        if (!isValid())
          return;
        int dbversionId = dbversionsList.get(dbversion.getSelectionIndex()).getId();
        int templateId = db.addOrEditTemplate(title.getText(), dbversionId,
                defaultButton.getSelection(), result != null ? ((Template)result).getId(): -1);
        result = new Template(templateId, title.getText(), dbversionId, defaultButton.getSelection());
        DosboxVersion selectedDBVersion = dbversionsList.get(dbversion.getSelectionIndex());
        Configuration dosboxConf = new Configuration(selectedDBVersion.getConfigFile());
        getConfigurationBySettings(templateConf);
        templateConf.substract(dosboxConf);
        templateConf.saveToFile(MainWindow.TEMPLATES_DIR + File.separatorChar +
        		templateId + MainWindow.CONF_EXT);
        shell.close();
      }
    });

    final Button cancelButton = new Button(composite_7, SWT.NONE);
    final RowData rowData_1 = new RowData();
    rowData_1.width = 80;
    cancelButton.setLayoutData(rowData_1);
    cancelButton.setText("Cancel");
    cancelButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        result = null;
        shell.close();
      }
    });
    
    // init values
    for (DosboxVersion dbv: dbversionsList) dbversion.add(dbv.getTitle());
    
    if (result != null) {
      // meaning we are essentially editing an existing template
      // so we need to set previous values
      title.setText(((Template)result).getTitle());
      defaultButton.setSelection(((Template)result).isDefault());
      int tableIndexVersionIndex = ListUtils.findTIDosboxVersion(dbversionsList, ((Template)result).getdbversionId()); 
      dbversion.select(tableIndexVersionIndex);
      DosboxVersion selectedDBVersion = dbversionsList.get(tableIndexVersionIndex);
      templateConf = new Configuration(selectedDBVersion.getConfigFile());
      templateConf.parseConfigurationFile(MainWindow.TEMPLATES_DIR + File.separatorChar +
      		((Template)result).getId() + MainWindow.CONF_EXT);
      selectSettingsByConfiguration(templateConf);
    } else {
      // set default values for new template
      int defaultDosboxVersionIndex = ListUtils.findTIDefaultDosboxVersion(dbversionsList);
      dbversion.select(defaultDosboxVersionIndex);
      DosboxVersion defaultDBVersion = dbversionsList.get(defaultDosboxVersionIndex);
      Configuration dosboxConf = new Configuration(defaultDBVersion.getConfigFile());
      templateConf = new Configuration(dosboxConf);
      selectSettingsByConfiguration(templateConf);
      // enabled 'exit afterwards' hardcoded, by default
      exit.setSelection(true);
    }
  }
  
  private String getAutoexecSectionBySettings() {
    return new Autoexec(mountingpoints.getItems(), false, "", "", "", "", "",
    		booterButton.getSelection()? "file": "", "", exit.getSelection() ).toString(false);
  }
    
  private void selectSettingsByAutoexec(Configuration templateConf) {
    Autoexec autoexec = new Autoexec(templateConf.getValue(Configuration.AUTOEXEC, Configuration.AUTOEXEC));
    exit.setSelection(autoexec.isExit());
    if (mountingpoints.getItemCount() == 0) {
      mountingpoints.setItems(autoexec.getMountingpoints());
      mountingpoints.setSelection(0);
    }
    dosButton.setSelection(!autoexec.isBooter());
    booterButton.setSelection(autoexec.isBooter());
  }
  
  private void selectSettingsByConfiguration(Configuration templateConf) {
    String priority = templateConf.getValue("sdl", "priority");
    String[] priorities = priority.split(",");
    if (priorities != null && priorities.length >= 2) {
      priority_active.setText(priorities[0]);
      priority_inactive.setText(priorities[1]);
    }
    waitonerror.setSelection(Boolean.valueOf(templateConf.getValue("sdl", "waitonerror")).booleanValue());
    language.setText(templateConf.getValue("dosbox", "language"));
    captures.setText(templateConf.getValue("dosbox", "captures"));
    output.setText(templateConf.getValue("sdl", "output"));
    frameskip.setText(templateConf.getValue("render", "frameskip"));
    scaler.setText(templateConf.getValue("render", "scaler"));
    pixelshader.setText(templateConf.getValue("sdl", "pixelshader"));
    pixelshader.setEnabled(templateConf.hasValue("sdl", "pixelshader"));
    fullscreen.setSelection(Boolean.valueOf(templateConf.getValue("sdl", "fullscreen")).booleanValue());
    fulldouble.setSelection(Boolean.valueOf(templateConf.getValue("sdl", "fulldouble")).booleanValue());
    fullresolution.setText(templateConf.getValue("sdl", "fullresolution"));
    windowresolution.setText(templateConf.getValue("sdl", "windowresolution"));
    aspect.setSelection(Boolean.valueOf(templateConf.getValue("render", "aspect")).booleanValue());
    overscan.setText(templateConf.getValue("sdl", "overscan"));
    overscan.setEnabled(templateConf.hasValue("sdl", "overscan"));
    svgachipset.setText(templateConf.getValue("vga", "svgachipset"));
    svgachipset.setEnabled(templateConf.hasValue("vga", "svgachipset"));
    videoram.setText(templateConf.getValue("vga", "videoram"));
    videoram.setEnabled(templateConf.hasValue("vga", "videoram"));
    vsyncmode.setText(templateConf.getValue("vsync", "vsyncmode"));
    vsyncmode.setEnabled(templateConf.hasValue("vsync", "vsyncmode"));
    vsyncrate.setText(templateConf.getValue("vsync", "vsyncrate"));
    vsyncrate.setEnabled(templateConf.hasValue("vsync", "vsyncrate"));
    glide.setSelection(Boolean.valueOf(templateConf.getValue("glide", "glide")).booleanValue());
    glide.setEnabled(templateConf.hasValue("glide", "glide"));
    glideport.setText(templateConf.getValue("glide", "port"));
    glideport.setEnabled(templateConf.hasValue("glide", "port") && glide.getSelection());
    machine.setText(templateConf.getValue("dosbox", "machine"));
    core.setText(templateConf.getValue("cpu", "core"));
    cycles.setText(templateConf.getValue("cpu", "cycles"));
    cycles_up.setText(templateConf.getValue("cpu", "cycleup"));
    cycles_down.setText(templateConf.getValue("cpu", "cycledown"));
    timesynched.setSelection(Boolean.valueOf(templateConf.getValue("cpu", "timesynched")).booleanValue());
    timesynched.setEnabled(templateConf.hasValue("cpu", "timesynched"));
    memsize.setText(templateConf.getValue("dosbox", "memsize"));
    xms.setSelection(Boolean.valueOf(templateConf.getValue("dos", "xms")).booleanValue());
    ems.setSelection(Boolean.valueOf(templateConf.getValue("dos", "ems")).booleanValue());
    umb.setText(templateConf.getValue("dos", "umb"));
    nosound.setSelection(Boolean.valueOf(templateConf.getValue("mixer", "nosound")).booleanValue());
    rate.setText(templateConf.getValue("mixer", "rate"));
    mt32rate.setText(templateConf.getValue("midi", "mt32rate"));
    mt32rate.setEnabled(templateConf.hasValue("midi", "mt32rate"));
    blocksize.setText(templateConf.getValue("mixer", "blocksize"));
    prebuffer.setText(templateConf.getValue("mixer", "prebuffer"));
    mpu401.setText(templateConf.getValue("midi", "mpu401"));
    device.setText(templateConf.getValue("midi", "device"));
    midi_config.setText(templateConf.getValue("midi", "config"));
    sbtype.setText(templateConf.getValue("sblaster", "sbtype"));
    oplmode.setText(templateConf.getValue("sblaster", "oplmode"));
    oplrate.setText(templateConf.getValue("sblaster", "oplrate"));
    sbbase.setText(templateConf.getValue("sblaster", "sbbase"));
    irq.setText(templateConf.getValue("sblaster", "irq"));
    dma.setText(templateConf.getValue("sblaster", "dma"));
    hdma.setText(templateConf.getValue("sblaster", "hdma"));
    mixer.setSelection(Boolean.valueOf(templateConf.getValue("sblaster", "mixer")).booleanValue());
    gus.setSelection(Boolean.valueOf(templateConf.getValue("gus", "gus")).booleanValue());
    gusrate.setText(templateConf.getValue("gus", "gusrate"));
    gusbase.setText(templateConf.getValue("gus", "gusbase"));
    irq1.setText(templateConf.getValue("gus", "irq1"));
    irq2.setText(templateConf.getValue("gus", "irq2"));
    dma1.setText(templateConf.getValue("gus", "dma1"));
    dma2.setText(templateConf.getValue("gus", "dma2"));
    ultradir.setText(templateConf.getValue("gus", "ultradir"));
    pcspeaker.setSelection(Boolean.valueOf(templateConf.getValue("speaker", "pcspeaker")).booleanValue());
    pcrate.setText(templateConf.getValue("speaker", "pcrate"));
    tandy.setText(templateConf.getValue("speaker", "tandy"));
    tandyrate.setText(templateConf.getValue("speaker", "tandyrate"));
    disney.setSelection(Boolean.valueOf(templateConf.getValue("speaker", "disney")).booleanValue());
    autolock.setSelection(Boolean.valueOf(templateConf.getValue("sdl", "autolock")).booleanValue());
    sensitivity.setText(templateConf.getValue("sdl", "sensitivity"));
    usescancodes.setSelection(Boolean.valueOf(templateConf.getValue("sdl", "usescancodes")).booleanValue());
    mapperfile.setText(templateConf.getValue("sdl", "mapperfile"));
    joysticktype.setText(templateConf.getValue("bios", "joysticktype"));
    serial1.setText(templateConf.getValue("serial", "serial1"));
    serial2.setText(templateConf.getValue("serial", "serial2"));
    serial3.setText(templateConf.getValue("serial", "serial3"));
    serial4.setText(templateConf.getValue("serial", "serial4"));
    ipx.setSelection(Boolean.valueOf(templateConf.getValue("ipx", "ipx")).booleanValue());
    
    selectSettingsByAutoexec(templateConf);
  }
  
  private void getConfigurationBySettings(Configuration templateConf) {
    templateConf.setValue("sdl", "priority", priority_active.getText() + "," + priority_inactive.getText());
    
    templateConf.setValue("sdl", "waitonerror", String.valueOf(waitonerror.getSelection()));
    templateConf.setValue("dosbox", "language", language.getText());
    templateConf.setValue("dosbox", "captures", captures.getText());
    templateConf.setValue("sdl", "output", output.getText());
    templateConf.setValue("render", "frameskip", frameskip.getText());
    templateConf.setValue("render", "scaler", scaler.getText());
    templateConf.updateValue("sdl", "pixelshader", pixelshader.getText());
    templateConf.setValue("sdl", "fullscreen", String.valueOf(fullscreen.getSelection()));
    templateConf.setValue("sdl", "fulldouble", String.valueOf(fulldouble.getSelection()));
    templateConf.setValue("sdl", "fullresolution", fullresolution.getText());
    templateConf.setValue("sdl", "windowresolution", windowresolution.getText());
    templateConf.setValue("render", "aspect", String.valueOf(aspect.getSelection()));
    templateConf.updateValue("sdl", "overscan", overscan.getText());
    templateConf.updateValue("vga", "svgachipset", svgachipset.getText());
    templateConf.updateValue("vga", "videoram", videoram.getText());
    templateConf.updateValue("vsync", "vsyncmode", vsyncmode.getText());
    templateConf.updateValue("vsync", "vsyncrate", vsyncrate.getText());
    templateConf.updateValue("glide", "glide", String.valueOf(glide.getSelection()));
    templateConf.updateValue("glide", "port", glideport.getText());
    templateConf.setValue("dosbox", "machine", machine.getText());
    templateConf.setValue("cpu", "core", core.getText());
    templateConf.setValue("cpu", "cycles", cycles.getText());
    templateConf.setValue("cpu", "cycleup", cycles_up.getText());
    templateConf.setValue("cpu", "cycledown", cycles_down.getText());
    templateConf.updateValue("cpu", "timesynched", String.valueOf(timesynched.getSelection()));
    templateConf.setValue("dosbox", "memsize", memsize.getText());
    templateConf.setValue("dos", "xms", String.valueOf(xms.getSelection()));
    templateConf.setValue("dos", "ems", String.valueOf(ems.getSelection()));
    templateConf.setValue("dos", "umb", umb.getText());
    templateConf.setValue("mixer", "nosound", String.valueOf(nosound.getSelection()));
    templateConf.setValue("mixer", "rate", rate.getText());
    templateConf.updateValue("midi", "mt32rate", mt32rate.getText());
    templateConf.setValue("mixer", "blocksize", blocksize.getText());
    templateConf.setValue("mixer", "prebuffer", prebuffer.getText());
    templateConf.setValue("midi", "mpu401", mpu401.getText());
    templateConf.setValue("midi", "device", device.getText());
    templateConf.setValue("midi", "config", midi_config.getText());
    templateConf.setValue("sblaster", "sbtype", sbtype.getText());
    templateConf.setValue("sblaster", "oplmode", oplmode.getText());
    templateConf.setValue("sblaster", "sbbase", sbbase.getText());
    templateConf.setValue("sblaster", "irq", irq.getText());
    templateConf.setValue("sblaster", "dma", dma.getText());
    templateConf.setValue("sblaster", "hdma", hdma.getText());
    templateConf.setValue("sblaster", "mixer", String.valueOf(mixer.getSelection()));
    templateConf.setValue("gus", "gus", String.valueOf(gus.getSelection()));
    templateConf.setValue("gus", "gusrate", gusrate.getText());
    templateConf.setValue("gus", "gusbase", gusbase.getText());
    templateConf.setValue("gus", "irq1", irq1.getText());
    templateConf.setValue("gus", "irq2", irq2.getText());
    templateConf.setValue("gus", "dma1", dma1.getText());
    templateConf.setValue("gus", "dma2", dma2.getText());
    templateConf.setValue("gus", "ultradir", ultradir.getText());
    templateConf.setValue("speaker", "pcspeaker", String.valueOf(pcspeaker.getSelection()));
    templateConf.setValue("speaker", "pcrate", pcrate.getText());
    templateConf.setValue("speaker", "tandy", tandy.getText());
    templateConf.setValue("speaker", "tandyrate", tandyrate.getText());
    templateConf.setValue("speaker", "disney", String.valueOf(disney.getSelection()));
    templateConf.setValue("sdl", "autolock", String.valueOf(autolock.getSelection()));
    templateConf.setValue("sdl", "sensitivity", sensitivity.getText());
    templateConf.setValue("sdl", "usescancodes", String.valueOf(usescancodes.getSelection()));
    templateConf.setValue("sdl", "mapperfile", mapperfile.getText());
    templateConf.setValue("bios", "joysticktype", joysticktype.getText());
    templateConf.setValue("serial", "serial1", serial1.getText());
    templateConf.setValue("serial", "serial2", serial2.getText());
    templateConf.setValue("serial", "serial3", serial3.getText());
    templateConf.setValue("serial", "serial4", serial4.getText());
    templateConf.setValue("ipx", "ipx", String.valueOf(ipx.getSelection()));
    
    templateConf.setValue(Configuration.AUTOEXEC, Configuration.AUTOEXEC, getAutoexecSectionBySettings());
  }
  
  private boolean isValid() {
    GeneralPurposeDialogs.initErrorDialog();
    if (title.getText().equals(""))
      GeneralPurposeDialogs.addError("You must enter a template title", title, infoTabItem);
    return !GeneralPurposeDialogs.displayErrorDialog(shell);
  }
}
